#
# Copyright (c) 1996 Shigio Yamaguchi. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
#	Pathop.pm
	$VERSION = '2.2';
#					10-Nov-1996	Shigio Yamaguchi
#
package Pathop;

require 5.002;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(exppath globpath cshpath);
#
# exppath: expand braced patterns {a,b} in a path
#
# Interface:
#	i)	$path	path include braced patterns
#	r)		expanded paths list
#
#	Note: Exppath doesn't check whether the path exist or not.
#
# Example: 
#	use Pathop;
#	@path = exppath('/usr/{src,local}/lib');
#	would yield @path eq ('/usr/src/lib', '/usr/local/lib')
#
# Version 1.2  10-Aug-1996: Shigio Yamaguchi <shigio@wafu.netgate.net>
#
sub exppath {
    my($path) = @_;
    my(@extends, @ex_right, $left, $match, $right, $item, $ext);

    $path =~ s/{}//g;
    unless ($path =~ /{[^}]+}/) {
	return $path;
    }
    $left = $`; $match = $&; $right = $';
    chop($match = substr($match, 1));
    @ex_right   = exppath($right);
    foreach $item (split(/,/, $match)) {
	foreach $ext (@ex_right) {
	    push(@extends, "$left$item$ext");
	}
    }
    @extends;
}
#
# globpath: expand glob patterns (*,?,[]) in a path
#
# Interface:
#	i)	$path	path include glob patterns
#	r)		expanded paths list
#
# Example: 
#	use Pathop;
#	@path = globpath('/usr/[ls]*/l?b');
#	would yield @path eq ('/usr/local/lib', '/usr/src/lib')
#
# Version 1.0  10-Aug-1996: Shigio Yamaguchi <shigio@wafu.netgate.net>
#
sub globpath {
    my($path) = @_;
    my($glob, @match, @extends, $left, $match, $right);

    foreach (split(/\//, $path)) {
	if (/[*?]/ || /\[.+\]/) {
	    $glob = $_;
	    last;
	}
    }
    if ($glob eq '') {
	return ((-e $path) ? ($path) : ());
    }
    $glob =~ s/([][?*^])/\\\1/g;
    if ($path =~ /$glob/) {
	$left = $`; $match = $&; $right = $';
    } else {
	warn "internal error.";
	return ();
    }
    $match =~ s/\./\\./g;
    $match =~ s/\?/./g;
    $match =~ s/\*/.*/g;
    opendir(DIR, ($left eq '') ? '.' : $left) || return ();
    foreach $p (readdir(DIR)) {
	next if ($p =~ /^\./ && $match !~ /^\\./);
	push(@match, $p) if ($p =~ /^$match$/);
    }
    closedir(DIR);
    foreach $m (sort @match) {
	if ($right eq '') {
	    push(@extends, "$left$m");
	    next;
	}
	foreach $p (globpath("$left$m$right")) {
	    next if ($p eq '');
	    push(@extends, $p);
	}
    }
    @extends;
}
#
# cshpath: expand cshell patterns in a path
#
# Interface:
#	i)	$path	path include cshell patterns
#	r)		expanded paths list
#
# Example: 
#	use Pathop;
#	@path = cshpath('~user/{s?c,l*}/lib');
#	would yield @path eq ('/home/user/src/lib', '/home/user/local/lib')
#
# Version 1.0  10-Aug-1996: Shigio Yamaguchi <shigio@wafu.netgate.net>
#
sub cshpath {
    my($path) = @_;
    my(@extends);

    if ($path =~ /~(\w+)/) {
	$dir = (getpwnam($1))[7];
	if ($dir eq '') {
	    warn "User '$1' not found.";
	    return ();
	}
	$path =~ s/~\w+/$dir/;
    } elsif ($path =~ /~/) {
	$path =~ s/~/$ENV{'HOME'}/;
    }
    foreach $e (exppath($path)) {
	foreach $g (globpath($e)) {
	    push(@extends, $g);
	}
    }
    @extends;
}

1;

__END__

=head1 NAME

exppath - expand braced patterns in a path

globpath - expand glob patterns in a path

cshpath - expand cshell patterns in a path

=head1 SYNOPSIS

    use Pathop;

    @paths = exppath($path)
    @paths = globpath($path)
    @paths = cshpath($path)

=head1 DESCRIPTION

=over 4

=item exppath

This routine expand a path which include braced patterns like
'{src,obj}' into separate strings.
It's a part of c-shell's file name expansion.

=item globpath

This routine expand a path which include glob patterns like
'*', '?', '[...]' into separate strings.
It's a part of c-shell's file name expansion.

=item cshpath

This routine expand a path which include cshell patterns like
'*', '?', '[...]', '{a,b}', '~', '~user' into separate strings.
It's maybe equal to c-shell's file name expansion.

=back

=head1 EXAMPLES

=over 4

=item exppath

    @paths = exppath('/usr/{src,obj}/prog');

would yield

    @paths eq ('/usr/src/prog', '/usr/obj/prog')

=item globpath

    @paths = globpath('/[Uu]sr/*/?tc');

would yield

    @paths eq ('/usr/local/etc' '/usr/src/etc')

=item cshpath

    @paths = cshpath('/[Uu]*/{s?c,lo?al}/?tc');

would yield

    @paths eq ('/usr/local/etc' '/usr/src/etc')

=back

=head1 AUTHOR

Shigio Yamaguchi <shigio@wafu.netgate.net>

=cut
